from django.core.management.base import BaseCommand
import subprocess
from pgrunner import bin_path

from pgrunner.commands import get_port, ensure_stopped, ensure_started


class Command(BaseCommand):
    help = 'Run pg_restore with proper arguments'

    def run_from_argv(self, argv):
        running = None

        pg_cmd = bin_path('pg_restore')
        port = get_port()
        if '--help' in argv:
            cmd = [pg_cmd, '--help']
        else:
            cmd = [pg_cmd, '-p', str(port), '-h', '127.0.0.1', 'django', '-O']
            cmd.extend(argv[2:])
            running = ensure_started()
        print(' '.join(cmd))
        subprocess.call(cmd)

        if not running and running is not None:
            ensure_stopped()




from django.core.management.base import BaseCommand
import subprocess
from pgrunner import bin_path

from pgrunner.commands import get_port, CURRENT

class Command(BaseCommand):
    help = 'Run pg_ctl with correct data dir'

    def run_from_argv(self, argv):
        pg_ctl = bin_path('pg_ctl')
        if '--help' in argv:
            cmd = [pg_ctl, '--help']
        else:
            cmd = [pg_ctl, '-D', CURRENT]
            cmd.extend(argv[2:])
        print(' '.join(cmd))
        subprocess.call(cmd)




from optparse import make_option
from django.core.management.base import BaseCommand, CommandError
import subprocess
from os.path import join, exists

from pgrunner.commands import ROOT, CURRENT, \
    ensure_stopped, ensure_started, activate_clone


class Command(BaseCommand):
    help = 'Create a named snapshot of the database'
    args = '<snapshot-name>'
    option_list = BaseCommand.option_list + (
        make_option('--activate',
            action='store_true',
            dest='activate',
            default=False,
            help='Immediately activate the snapshot after creating it'),
    )

    def handle(self, *args, **options):
        if not args:
            raise CommandError("No snapshot name specified")
        snapshot = args[0]
        snapshot_path = join(ROOT, snapshot)
        if exists(snapshot_path):
            raise CommandError("Snapshot already exists: {0}".format(snapshot_path))

        running = ensure_stopped(verbose=True)

        # rsync must be in the system PATH
        cmd = ['rsync', '-aP', '--progress', CURRENT + '/.', snapshot_path]
        print(' '.join(cmd))
        if subprocess.call(cmd) > 0:
            raise CommandError("rsync failed")

        if options['activate']:
            print('Activating the snapshot.')
            activate_clone(snapshot)

        if running:
            ensure_started(verbose=True)
            print("Server restarted in background.")

        print()
        print("Snapshot done.")

        if not options['activate']:
            print()
            print("To activate:")
            print()
            print("    ./manage.py pg_activate", snapshot)





from django.core.management.base import BaseCommand, CommandError
import subprocess
import os
from os.path import join, exists
import time

from pgrunner.commands import ROOT, activate_clone, \
    ensure_stopped, ensure_started


class Command(BaseCommand):
    help = 'Activate a snapshot'
    args = '<snapshot-name>'

    def list_snapshots(self):
        print("Available snapshots:")
        for name in os.listdir(ROOT):
            if name == 'current' or name.startswith('.'):
                continue
            if exists(join(ROOT, name, 'postgresql.conf')):
                print('-', name)

    def handle(self, *args, **options):
        if not args:
            self.list_snapshots()
            return

        snapshot = args[0]
        if snapshot == 'current':
            raise CommandError("Invalid snapshot name")

        if not exists(join(ROOT, snapshot)):
            self.list_snapshots()
            raise CommandError("Snapshot not found: {0}".format(snapshot))

        running = ensure_stopped(verbose=True)

        activate_clone(snapshot)

        if running:
            ensure_started(verbose=True)
            print("Server restarted in background.")

        print()
        print("Snapshot activated.")




from django.core.management.base import BaseCommand, CommandError
import subprocess
from pgrunner import bin_path

from pgrunner.commands import get_port, CURRENT

class Command(BaseCommand):
    help = 'Start PostgreSQL in foreground'

    def handle(self, *args, **options):
        print("Starting server on port", get_port())
        cmd = [bin_path('postgres'), '-D', CURRENT]
        print(' '.join(cmd))
        subprocess.call(cmd)



import os
import sys
from django.core.exceptions import ImproperlyConfigured

DEFAULT_NAME = 'pgrunnerdb'
DEFAULT_ROOT = os.path.join(os.getcwd(), DEFAULT_NAME)
ROOT = DEFAULT_ROOT
BIN = None

def bin_path(name):
    """Convert a binary name to a full path if needed

    :param name: name of binary
    :type name: str
    :return: full path to binary, or just the name if in PATH
    :rtype: str
    """
    if BIN:
        return os.path.join(BIN, name)
    else:
        return name


def settings(settings_locals, start=True):
    """Register settings for PostgreSQL dev database

    This will set `DATABASES['default']` to the right values for the development
    environment, and start the database server if needed.

    Usage in `settings.py`::

        # Make sure to do this after DATABASES and any settings for this module
        import pgrunner
        pgrunner.settings(locals())

    :param settings_locals: locals() in settings.py
    :type settings_locals: dict
    """
    global ROOT, BIN
    ROOT = settings_locals.get('PGRUNNER_ROOT', None)
    if not ROOT:
        ROOT = DEFAULT_ROOT
    BIN = settings_locals.get('PGRUNNER_BIN', None)
    if BIN and not (os.path.isdir(BIN) and os.path.exists(bin_path('initdb'))):
        raise ImproperlyConfigured(
            "PGRUNNER_BIN setting incorrect: "
            "no folder or initdb not found in {0}".format(BIN))

    db_exists = os.path.exists(ROOT)

    from pgrunner.commands import get_port, ensure_started

    if not db_exists:
        print("PostgreSQL dev root does not exist:", ROOT)
        print("Not changing any settings.")
        print("Please run `./manage.py pg_init` first!")

    if db_exists:
        if not 'DATABASES' in settings_locals:
            settings_locals['DATABASES'] = {}

        settings_locals['DATABASES']['default'] = {
            'ENGINE': 'django.db.backends.postgresql_psycopg2',
            'NAME': 'django',
            'USER': '',
            'PASSWORD': '',
            'HOST': '127.0.0.1',
            'PORT': get_port()
        }

    running_own_command = False
    if len(sys.argv) > 1 and sys.argv[0].endswith('manage.py') \
            and sys.argv[1].startswith('pg_'):
        running_own_command = True

    help = len(sys.argv) == 1 and sys.argv[0].endswith('manage.py')

    if db_exists and start and not running_own_command and not help:
        ensure_started(verbose=True)



import os
import sys
from os.path import join
import re

import subprocess
import time

from pgrunner import ROOT

# Default database (in the future we will put clones in the ROOT too)
DEFAULT = join(ROOT, 'default')

# Active database
CURRENT = join(ROOT, 'current')

HELP = """Useful commands
    ./manage.py pg_run           - Run PostgreSQL server in foreground
    ./manage.py pg_psql          - Start psql with right parameters
    ./manage.py pg_ctl start     - Start server in background
    ./manage.py pg_ctl stop      - Stop server in background
    ./manage.py pg_ctl status    - Check if the server is running
    ./manage.py pg_snapshot foo  - Create a copy of all current database data
    ./manage.py pg_activate foo  - Activate snapshot 'foo'
"""

def activate_clone(snapshot='default'):
    """Activate a certain snapshot by name

    :param snapshot: name of snapshot
    :type snapshot: str
    """
    # TODO: restrict snapshot names to [a-zA-Z0-9_-]+
    if os.pathsep in snapshot or '/' in snapshot:
        raise ValueError("Invalid snapshot name: {0}".format(snapshot))

    snapshot_path = join(ROOT, snapshot)
    if not os.path.isdir(snapshot_path):
        raise OSError("Not a snapshot directory: {0}".format(snapshot_path))

    current_path = join(ROOT, 'current')
    if os.path.exists(current_path):
        os.unlink(current_path)
    os.symlink(snapshot, current_path)


GET_PORT_RE = re.compile('(^|\n) *port *= *([0-9]+)')
SET_PORT_RE = re.compile('(^|\n)#? *port *= *([0-9]+)')


def get_port():
    """Returns the port the server listens on

    :return: port number
    :rtype: int
    """
    config_path = join(CURRENT, 'postgresql.conf')
    with open(config_path, 'r') as f:
        config = f.read()

    m = GET_PORT_RE.search(config, re.MULTILINE)
    if not m:
        port = 5432
    else:
        port = int(m.group(2))

    return port


def set_port(port):
    """Changes the postgresql config to use given port

    :param port: the port to listen on
    :type port: int
    """
    config_path = join(CURRENT, 'postgresql.conf')
    with open(config_path, 'r') as f:
        config = f.read()

    config = SET_PORT_RE.sub('\\1port = {0}'.format(port), config, re.MULTILINE)

    with open(config_path, 'w') as f:
        f.write(config)


def is_running():
    """Checks if the server is running.

    :return: is running?
    :rtype: bool
    """
    return os.path.exists(join(CURRENT, 'postmaster.pid'))


def ensure_stopped(verbose=False):
    """Ensures the database server is not running and stops it if needed.

    :param verbose: If set, info will be printed to stdout
    :type verbose: bool
    :return: indicates if the server was already running
    :rtype: bool
    """
    running = is_running()
    if running:
        if verbose:
            print("PostgreSQL server is running, shutting it down")
        cmd = ['pg_ctl', '-D', CURRENT, '-m', 'fast', 'stop']
        if verbose:
            print(' '.join(cmd))
        subprocess.call(cmd)
        for i in range(20):
            if not is_running():
                break
            time.sleep(0.5)
        if is_running():
            raise Exception("Server still running after 10 seconds")
    return running


def ensure_started(verbose=False):
    """Ensures the database server is running and starts it if needed.

    :param verbose: If set, info will be printed to stdout
    :type verbose: bool
    :return: indicates if the server was already running
    :rtype: bool
    """
    running = is_running()
    if not running:
        if verbose:
            print("PostgreSQL server is not running, starting it")
        cmd = ['pg_ctl', '-D', CURRENT, 'start']
        if verbose:
            print(' '.join(cmd))
        subprocess.call(cmd)
        for i in range(20):
            if is_running():
                break
            time.sleep(0.5)
        if not is_running():
            raise Exception("Server still not running after 10 seconds")
    return running




from django.core.management.base import BaseCommand, CommandError
import subprocess
from pgrunner import bin_path

from pgrunner.commands import get_port

class Command(BaseCommand):
    help = 'Run psql with correct database'

    def run_from_argv(self, argv):
        port = get_port()
        cmd = [bin_path('psql'), '-p', str(port), '-h', '127.0.0.1', 'django']
        cmd.extend(argv[2:])
        print(' '.join(cmd))
        subprocess.call(cmd)



import os
import subprocess
import random
from os.path import join
import time

from django.core.management.base import BaseCommand, CommandError

from pgrunner import DEFAULT_NAME, bin_path
from pgrunner.commands import ROOT, DEFAULT, activate_clone, \
    set_port, get_port, CURRENT, HELP

SETTINGS = """
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'django',
        'USER': '',
        'PASSWORD': '',
        'HOST': '127.0.0.1',
        'PORT': $port$
    }
}
"""

PORT_MIN = 15000
PORT_MAX = 16000

class Command(BaseCommand):
    help = 'Initializes a new local PostgreSQL database'

    def handle(self, *args, **options):
        if os.path.isdir(ROOT):
            raise CommandError('Directory already exists: {0}'.format(ROOT))

        print("Creating new PostgreSQL root for development in {0}".format(ROOT))
        os.mkdir(ROOT)

        cmd = [bin_path('initdb'), DEFAULT]
        print(' '.join(cmd))
        if subprocess.call(cmd) > 0:
            raise CommandError('Error creating database in {0}'.format(DEFAULT))
        activate_clone('default')

        print("New database in {0}".format(DEFAULT))

        # TODO: create option to pick a specific one
        port = random.randint(PORT_MIN, PORT_MAX)
        set_port(port)
        if get_port() != port:
            raise CommandError('Setting port failed')
        print("Port set to {0} (can be changed in {1})".format(
            port, join(DEFAULT, 'postgresql.conf')))

        # TODO: number of requested standby connections exceeds
        #       max_wal_senders (currently 0)
        print("Enabling replication permissions in pg_hba.conf")
        pg_hba_path = join(CURRENT, 'pg_hba.conf')
        with open(pg_hba_path, 'r') as f:
            pg_hba = f.read().split('\n')
        with open(pg_hba_path, 'w') as f:
            for line in pg_hba:
                if len(line) > 2 and line[0] == '#' and line[1] != ' ':
                    # Strip comment character
                    line = line[1:]
                f.write(line + '\n')

        print("Starting server to create database 'django'")
        cmd = [bin_path('pg_ctl'), '-D', CURRENT, 'start']
        print(' '.join(cmd))
        if subprocess.call(cmd) > 0:
            raise CommandError('Error starting database')

        print("Pausing 3s so that the database can start up")
        time.sleep(3)

        print("Creating database 'django'")
        cmd = [bin_path('createdb'), '-p', str(port), '-h', '127.0.0.1', 'django']
        print(' '.join(cmd))
        if subprocess.call(cmd) > 0:
            raise CommandError('Error creating database')

        print("Stopping server")
        cmd = [bin_path('pg_ctl'), '-D', CURRENT, 'stop']
        print(' '.join(cmd))
        if subprocess.call(cmd) > 0:
            raise CommandError('Error stopping database')

        print()
        print("Example configuration for settings.py:")
        settings = SETTINGS.replace('$port$', str(port))
        print(settings)
        with open(join(ROOT, 'settings.py'), 'w') as f:
            f.write(settings)

        #print()
        #print("These settings have been written to postgresdb/settings.py")
        #print("At the end of your settings.py or local settings file, add:")
        #print()
        #print("from {0}.settings import *".format(DEFAULT_NAME))
        print()
        print("Simply add this at the end of your settings file to")
        print("automatically set the right database settings and start the ")
        print("database if needed:")
        print()
        print('import pgrunner')
        print('pgrunner.settings(locals())')
        print()
        print(HELP)




import os
from setuptools import setup, find_packages

version = '0.2.1'

# Will be replaced by the contents of README.rst, if available
desc = """Create and autorun a local PostgreSQL development database for your Django project."""

# This file is not shipped in eggs for some reason..
if os.path.isfile('README.rst'):
    desc = open('README.rst').read()

setup(name='django-pgrunner',
      version=version,
      description="Create and autorun a PostgreSQL development database for your Django project",
      long_description=desc,
      classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Topic :: Database',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Operating System :: MacOS :: MacOS X',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
      ], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='django postgres postgresql runner pgrunner',
      author='Konrad Wojas',
      author_email='konrad@wojas.nl',
      url='https://github.com/wojas/django-pgrunner/',
      license='LICENSE',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests', 'test_project']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
        'Django >= 1.4'
      ],
      entry_points="""
      """,
      )
